<?php

/**
 * Combine data array and structures together and walk through the options.
 * Automatically calls these abstract functions:
 * 
 * _beforeContainer
 * _afterContainer
 * _oneOption
 * 
 * You can inherit them, it's useful for printing, querying options and other
 * stuff.
 * 
 * @author FRESHFACE
 * @since 0.1
 * 
 */

abstract class ffOptionsWalker extends ffBasicObject {
/******************************************************************************/
/* VARIABLES AND CONSTANTS
/******************************************************************************/
	private $_optionsArrayData = null;
	private $_optionsStructure = null;
	
	private $_currentRoute = array();
	private $_currentRouteCount = 0;
	protected $_walkElements = true;
/******************************************************************************/
/* CONSTRUCT AND PUBLIC FUNCTIONS
/******************************************************************************/
	public function __construct( $optionsArrayData = null, $optionsStructure = null ) {
		$this->_setOptionsArrayData( $optionsArrayData );
		$this->_setOptionsStructure($optionsStructure);
	}
	
	public function setOptionsArrayData( $optionsArrayData ) {
		$this->_setOptionsArrayData($optionsArrayData);
	}
	
	public function setOptionsStructure( $optionsStructure ) {
		$this->_setOptionsStructure($optionsStructure);
	}
	
/******************************************************************************/
/* ABSTRACT FUNCTION
/******************************************************************************/	
	protected abstract function _beforeContainer( $item );
	protected abstract function _afterContainer( $item );
	protected abstract function _oneOption( $item );
	protected function _oneElement( ffOneElement $item ) {
		
	}
/******************************************************************************/
/* PRIVATE FUNCTIONS
/******************************************************************************/
	/**
	 * Start walkings
	 */
	protected function _walk() {
		
		if( get_class($this->_getOptionsStructure()) == 'ffOneStructure' ) {
			foreach( $this->_getOptionsStructure()->getData() as $oneItem ) {
				$this->_walkItem( $oneItem );
			}
		}
		else { 
			$this->_walkItem( $this->_getOptionsStructure()->getData() );
		}
	}
	/**
	 * Deciding if its container ( containes other options), or directly
	 * option.
	 * @param ffOneOption|ffOneStructure $item
	 */
	private function _walkItem( $item ) {
		if( $this->_isItemContainer($item) ) {
			$this->_walkContainer( $item );
		} else if( $item instanceof ffOneOption ) {
			$this->_walkOption( $item );
		} else {
			$this->_walkElement( $item );
		}
	}
	
	private function _walkContainer( $item ) {
		if( $this->_isContainerRepeatable($item) ) {
			$this->_walkRepeatableContainer( $item );
		} else {
			$this->_walkNonRepeatableContainer( $item );
		}
	}
	
	private function _walkNonRepeatableContainer( $item ) {
		$this->_addRoute( $item->getId() );
			$this->_beforeContainer($item);
			foreach( $item->getData() as $childItem ) {
				$this->_walkItem( $childItem );
			}
			$this->_afterContainer($item);
		$this->_removeRoute();
	}
	
	/**
	 * Repeatable containers has specific data saving structure. It's like this:
	 * 
	 * [repeatableContainer][0][optionName]
	 * [repeatableContainer][1][optionName]
	 * 
	 * See the numbers, they are here to identify the repeatabling.
	 * @param ffOneStructure $item
	 */
	private function _walkRepeatableContainer( $item ) {
		// TODO there MIGHT be problem when repeatables are 0,1,5,6,9,10
		$this->_addRoute( $item->getId() );
			$this->_beforeContainer($item);
			$currentRepeatableRoute = $this->_getCurrentRouteValue();
			$repeatableCount = count( $currentRepeatableRoute );
			if( $repeatableCount == 0) $repeatableCount = 1;
			for( $i = 0; $i < $repeatableCount; $i++ ) {
				$this->_addRoute( $i );
					foreach( $item->getData() as $childItem ) {
						$this->_walkItem( $childItem );
					}
				$this->_removeRoute();
			}
			$this->_afterContainer($item);
		$this->_removeRoute();
	}
	
	private function _walkOption( $item ) {
		$this->_addRoute( $item->getId() );
			$item->setValue( $this->_getOptionValue($item) );
			$this->_oneOption($item);
		$this->_removeRoute();
	}
	
	private function _walkElement( $item ) {
		if( !$this->_walkElements ) {
			return false;
		}
		
		$this->_addRoute( $item->getId() );
			$this->_oneElement($item);
		$this->_removeRoute();
	}
	
	protected function _getOptionValue( $item ) {
		$currentValue = ($this->_getCurrentRouteValue());
		$itemValue = $item->getDefaultValue();
		
		( $currentValue == null ) ? $toReturn = $itemValue : $toReturn = $currentValue;
		return $toReturn;
	}
	
	protected function _getCurrentRouteValue() {
		return $this->_getDataValue( $this->_getRoute() );
	}
	
	private function _getDataValue( $route ) {
		$data = &$this->_optionsArrayData;
		foreach( $route as $onePath ) {
			if( isset( $data[ $onePath ] ) ) {
				$data = &$data[$onePath];
			} else {
				return null;
			}
		}
		return $data;
	}
	
	private function _isItemContainer( $item ) {
		//$containerClasses = array('ffOneStructure', 'ffOneSection');
		//$className = get_class( $item );
		return $item->isContainer();//in_array( $className, $containerClasses );
	}
	
	private function _isContainerRepeatable( $item ) {
		$type = $item->getType();
		if( ffOneSection::TYPE_REPEATABLE === $type ) {
			return true; 
		} else {
			return false;
		}
	}
	
/******************************************************************************/
/* SETTERS AND GETTERS
/******************************************************************************/
	/********** CURRENT ROUTE **********/
	protected function _addRoute( $routeName ) {
		$this->_currentRoute[] = $routeName;
	}
	
	protected function _removeRoute() {
		array_pop( $this->_currentRoute );
	}
	
	protected function _getRoute() {
		return $this->_currentRoute;
	}
	
	/********** OPTIONS ARRAY DATA **********/
	protected function _setOptionsArrayData( $optionsArrayData ) {
		$this->_optionsArrayData = $optionsArrayData;
	}
	/**
	 * @return array;
	 */
	protected function _getOptionsArrayData() {
		return $this->_optionsArrayData;
	}
	
	
	/********** OPTIONS STRUCTURE **********/
	/**
	 * 
	 * @param ffOneStructure|ffOneSection $optionsStructure
	 */
	protected function _setOptionsStructure( $optionsStructure ) {
		$this->_optionsStructure = $optionsStructure;
	}
	
	protected function _getOptionsStructure() {
		return $this->_optionsStructure;
	}
}